/************************************************

Made for timing laser machine extractor after use in Iurreta LHII vocational school in Iurreta, Basque Country

First Date: January 2024
Last Date: June 2024

Version: 0.80
Author: Alfredo Barrainkua
License: Public Domain
(c) 2024; Alfredo Barrainkua, Iurreta LHI3

Used PINs
  Time selectors        // PORT A
  Timer signal trigger  // D8 // PB.2 // PIN 5 ATtiny84
  Relay output          // D10 = PB.0 = PIN 2 ATtiny84

            ______________________________
           |                              |
Power..____|                              |____________

             ______________________
             |                     |
Signal ...___|                     |_____________

                ______________________
                |                     |
Output....._____|                     |______

             
t0:........__|t0|____
                                   
t1: .......________________________|t1|_______

************************************************/

#include <SoftwareSerial.h>
#include <avr/io.h>

#define SIGNAL_IS_ACTIVE (!digitalRead(timerSignal))
#define SIGNAL_IS_NOT_ACTIVE (digitalRead(timerSignal))

// Function Prototypes
void getOnOffTimeValues(void);

unsigned long int toONMillis = 0;
unsigned long int toOFFMillis = 0;

unsigned long int onDelayMillis = 0;
unsigned long int offDelayMillis = 0;

int timerSignal = 8; // PB.2 = PIN 5 ATtiny84
int relayPin = 10; // = PB.0 = PIN 2 ATtiny84

// -------- FUNCTIONS --------

// SETUP
void setup() {
  // Signal pin
  pinMode(timerSignal, INPUT_PULLUP);

  // Relay pin configured as output
  pinMode(relayPin, OUTPUT);
}  // setup()


// LOOP
void loop() {
  // Wait until timer signal activate (Negative logic)
  for(;digitalRead(timerSignal););

  getOnOffTimeValues();

  // ON DELAY Time
  for(onDelayMillis = millis(); ((millis() - onDelayMillis) < toONMillis) && (SIGNAL_IS_ACTIVE););

  // Activate RELAY
  if (SIGNAL_IS_ACTIVE) digitalWrite(relayPin, HIGH);

  // Wait while timer signal is active
  for(;SIGNAL_IS_ACTIVE;);

  // OFF DELAY Time
  for(offDelayMillis = millis(); ((millis() - offDelayMillis) < toOFFMillis) && (SIGNAL_IS_NOT_ACTIVE););

  // DEActivate RELAY
  if (SIGNAL_IS_NOT_ACTIVE) digitalWrite(relayPin, LOW);
}  // loop()



void getOnOffTimeValues(void){
  unsigned int toON = 0;
  unsigned int toOFF = 0;
  byte inputPortA;

  // All bits of port A configured as inputs (time selectors) (Data Direction Register A)
  DDRA = 0b00000000;
  // All pins of port A with PullUp
  PORTA = 0b11111111;

  delay(10);

  // Read digital port A
  inputPortA = ~PINA;
  toON = (unsigned int)(inputPortA & 0b00001111);
  toOFF = (unsigned int)((inputPortA >> 4) & 0b00001111);

  // selector number x10 seconds = x10000 millis
  toONMillis = toON * 10000;
  toOFFMillis = toOFF * 10000;
}
